%{
/*
 *  ippl.l - functions parsing the configuration file
 *
 *  Copyright (C) 1998 Etienne Bernard
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>

#include "filter.h"
#include "ippl.tab.h"

int line_number = 1;

%}

white     [ \t]+

octdigit  [0-7]
hexdigit  [0-9A-Fa-f]
decdigit  [0-9]

octchar   {octdigit}+
hexchar   {hexdigit}+
decchar   {decdigit}+

char      {octchar}|{hexchar}|{decchar}
integer   {decdigit}+

ip        {char}"."{char}"."{char}"."{char}

letter    [A-Za-z]
digit     [0-9]
letdig    {letter}|{digit}
letdighyp {letdig}|"-"
label     {letter}({letdighyp}*{letdig})?

hostmask  (({label}|"*")".")*({label}|"*")"."?

%%

{white}               { /* Ignore */ }
"#".*$                { /* Ignore */ }
"\n"                  { line_number++; return EOL; }

[rR][uU][nN]               return RUN;
[lL][oO][gG]               return LOG;
[iI][gG][nN][oO][rR][eE]   return IGNORE;
[fF][rR][oO][mM]           return FROM;
[tT][yY][pP][eE]           return TYPE;
[pP][oO][rR][tT]           return PORT;

[iI][cC][mM][pP]      return ICMP;
[tT][cC][pP]          return TCP;
[aA][lL][lL]          return ALL;

{integer}             {
                        yylval.longval = atol(yytext);
                        return NUMBER;
                      }

"-"|"--"              return RANGE;
"/"                   return SLASH;

"echo_reply"          { yylval.longval = 0; return NUMBER; }
"dest_unreach"        { yylval.longval = 3; return NUMBER; }
"src_quench"          { yylval.longval = 4; return NUMBER; }
"redirect"            { yylval.longval = 5; return NUMBER; }
"echo_req"            { yylval.longval = 8; return NUMBER; }
"router_advert"       { yylval.longval = 9; return NUMBER; }
"router_solicit"      { yylval.longval = 10; return NUMBER; }
"time_exceeded"       { yylval.longval = 11; return NUMBER; }
"param_problem"       { yylval.longval = 12; return NUMBER; }
"ts_req"              { yylval.longval = 13; return NUMBER; }
"ts_reply"            { yylval.longval = 14; return NUMBER; }
"info_req"            { yylval.longval = 15; return NUMBER; }
"info_reply"          { yylval.longval = 16; return NUMBER; }
"addr_mask_req"       { yylval.longval = 17; return NUMBER; }
"addr_mask_reply"     { yylval.longval = 18; return NUMBER; }

{ip}                  {
                        yylval.stringval = strdup(yytext);
                        return IP;
                      }

{hostmask}            {
                        if (yytext[strlen(yytext)-1] == '.')
                          yytext[strlen(yytext)-1] = '\0';
                        yylval.stringval = strdup(yytext);
                        return HOSTMASK;
                      }

.                     {
                        /* Otherwise, this is an error */
                        printf("Invalid keyword line %d\n", line_number);
                        exit(1);
                      }
%%

int yywrap() {
  return 1;
}
